@@ -170,6 +170,8 @@ public:
 					saw_last_read = ret.second;
 					if(ret.first && (*patsrcs_)[i]->rdid() < qUpto_) {
 						(*aligners_)[i]->setQuery((*patsrcs_)[i]);
+					} else if (ret.first) {
+						saw_last_read = true;
 					}
 				}
 			}
@@ -279,7 +281,7 @@ public:
 					// Get a new read
 					pair<bool, bool> ret = ps->nextReadPair();
 					saw_last_read = ret.second;
-					if(ps->rdid() < qUpto_ && ret.first) {
+					if(ret.first && ps->rdid() < qUpto_) {
 						if(ps->paired()) {
 							// Read currently in buffer is paired-end
 							(*alignersPE_)[0]->setQuery(ps);
@@ -291,6 +293,8 @@ public:
 							al = (*alignersSE_)[0];
 							seOrPe_[0] = true; // true = unpaired
 						}
+					} else if (ret.first) {
+						break;
 					}
 				}
 				first = false;
@@ -326,7 +330,7 @@ public:
 						// Get a new read
 						pair<bool, bool> ret = ps->nextReadPair();
 						saw_last_read = ret.second;
-						if(ps->rdid() < qUpto_ && ret.first) {
+						if (ret.first && ps->rdid() < qUpto_) {
 							if(ps->paired()) {
 								// Read currently in buffer is paired-end
 								(*alignersPE_)[i]->setQuery(ps);
@@ -336,6 +340,8 @@ public:
 								(*alignersSE_)[i]->setQuery(ps);
 								seOrPe_[i] = true; // true = unpaired
 							}
+						} else if (ret.first) {
+							break;
 						}
 					}
 				}
@@ -475,7 +481,6 @@ public:
 				colorExEnds,
 				snpPhred,
 				refs_,
-				ra.ebwt->rmap(),
 				ebwtFw,
 				ra.mms,                   // mismatch positions
 				ra.refcs,                 // reference characters for mms
@@ -861,8 +866,7 @@ protected:
 	            TIndexOffU tlen, // length of ref
 	            bool pairFw,   // whether the pair is being mapped to fw strand
 	            bool ebwtFwL,
-	            bool ebwtFwR,
-	            const ReferenceMap* rmap)
+	            bool ebwtFwR)
 	{
 		assert(gAllowMateContainment || upstreamOff < dnstreamOff);
 		TIndexOffU spreadL = rL.bot - rL.top;
@@ -889,7 +893,6 @@ protected:
 				colorExEnds,
 				snpPhred,
 				refs_,
-				rmap,
 				ebwtFwL,
 				rL.mms,                       // mismatch positions
 				rL.refcs,                     // reference characters for mms
@@ -924,7 +927,6 @@ protected:
 				colorExEnds,
 				snpPhred,
 				refs_,
-				rmap,
 				ebwtFwR,
 				rR.mms,                       // mismatch positions
 				rR.refcs,                     // reference characters for mms
@@ -951,12 +953,11 @@ protected:
 	            TIndexOffU upstreamOff, // offset for upstream mate
 	            TIndexOffU dnstreamOff, // offset for downstream mate
 	            TIndexOffU tlen, // length of ref
-	            bool pairFw,   // whether the pair is being mapped to fw strand
-	            const ReferenceMap* rmap)
+	            bool pairFw)   // whether the pair is being mapped to fw strand
 	{
 		return report(rL, rR, first, upstreamOff,
 		              dnstreamOff, tlen,
-		              pairFw, rL.ebwt->fw(), rR.ebwt->fw(), rmap);
+		              pairFw, rL.ebwt->fw(), rR.ebwt->fw());
 	}
 
 	/**
@@ -1098,8 +1099,7 @@ protected:
 				tlen,       // length of ref
 				!doneFw_,   // whether the pair is being mapped to fw strand
 				ebwtLFw,
-				ebwtRFw,
-				range.ebwt->rmap())) return true;
+				ebwtRFw)) return true;
 		}
 		return false;
 	}
@@ -1739,8 +1739,7 @@ protected:
 	            TIndexOffU tlen, // length of ref
 	            bool pairFw,   // whether the pair is being mapped to fw strand
 	            bool ebwtFwL,
-	            bool ebwtFwR,
-	            const ReferenceMap *rmap)
+	            bool ebwtFwR)
 	{
 		assert(gAllowMateContainment || upstreamOff < dnstreamOff);
 		TIndexOffU spreadL = rL.bot - rL.top;
@@ -1767,7 +1766,6 @@ protected:
 				colorExEnds,
 				snpPhred,
 				refs_,
-				rmap,
 				ebwtFwL,
 				rL.mms,                       // mismatch positions
 				rL.refcs,                     // reference characters for mms
@@ -1802,7 +1800,6 @@ protected:
 				colorExEnds,
 				snpPhred,
 				refs_,
-				rmap,
 				ebwtFwR,
 				rR.mms,                       // mismatch positions
 				rR.refcs,                     // reference characters for mms
@@ -1849,7 +1846,6 @@ protected:
 			colorExEnds,
 			snpPhred,
 			refs_,
-			r.ebwt->rmap(),
 			ebwtFw,
 			r.mms,                   // mismatch positions
 			r.refcs,                 // reference characters for mms
@@ -2034,8 +2030,7 @@ protected:
 				tlen,       // length of ref
 				pairFw,     // whether the pair is being mapped to fw strand
 				ebwtLFw,
-				ebwtRFw,
-				range.ebwt->rmap())) return true;
+				ebwtRFw)) return true;
 		}
 		return false;
 	}
