@@ -3,6 +3,8 @@
 
 #include <cassert>
 #include <cmath>
+#include <zlib.h>
+#include <sys/stat.h>
 #include <stdexcept>
 #include <vector>
 #include <string>
@@ -371,6 +373,7 @@ struct PerThreadReadBuf {
 	 */
 	void setReadId(TReadId rdid) {
 		rdid_ = rdid;
+		assert_neq(rdid_, std::numeric_limits<TReadId>::max());
 	}
 	
 	const size_t max_buf_; // max # reads to read into buffer at once
@@ -391,21 +394,10 @@ struct PerThreadReadBuf {
  */
 class PatternSource {
 public:
-	PatternSource(
-		const char *dumpfile = NULL) :
+	PatternSource() :
 		readCnt_(0),
-		dumpfile_(dumpfile),
 		mutex()
-	{
-		// Open dumpfile, if specified
-		if(dumpfile_ != NULL) {
-			out_.open(dumpfile_, ios_base::out);
-			if(!out_.good()) {
-				cerr << "Could not open pattern dump file \"" << dumpfile_ << "\" for writing" << endl;
-				throw 1;
-			}
-		}
-	}
+	{ }
 
 	virtual ~PatternSource() { }
 
@@ -436,19 +428,6 @@ public:
 
 protected:
 
-	/**
-	 * Dump the contents of the ReadBuf to the dump file.
-	 */
-	void dumpBuf(const Read& r) {
-		assert(dumpfile_ != NULL);
-		dump(out_, r.patFw,
-		     empty(r.qual) ? String<char>("(empty)") : r.qual,
-		     empty(r.name) ? String<char>("(empty)") : r.name);
-		dump(out_, r.patRc,
-		     empty(r.qualRev) ? String<char>("(empty)") : r.qualRev,
-		     empty(r.name) ? String<char>("(empty)") : r.name);
-	}
-
 	/**
 	 * Default format for dumping a read to an output stream.  Concrete
 	 * subclasses might want to do something fancier.
@@ -464,9 +443,6 @@ protected:
 	/// The number of reads read by this PatternSource
 	volatile uint64_t readCnt_;
 
-	const char *dumpfile_; /// dump patterns to this file before returning them
-	ofstream out_;         /// output stream for dumpfile
-
 	/// Lock enforcing mutual exclusion for (a) file I/O, (b) writing fields
 	/// of this or another other shared object.
 	MUTEX_T mutex;
@@ -480,10 +456,9 @@ protected:
  */
 class TrimmingPatternSource : public PatternSource {
 public:
-	TrimmingPatternSource(const char *dumpfile = NULL,
-	                      int trim3 = 0,
+	TrimmingPatternSource(int trim3 = 0,
 	                      int trim5 = 0) :
-		PatternSource(dumpfile),
+		PatternSource(),
 		trim3_(trim3), trim5_(trim5) { }
 protected:
 	int trim3_;
@@ -502,10 +477,9 @@ class VectorPatternSource : public TrimmingPatternSource {
 public:
 	VectorPatternSource(
 		const vector<string>& v,
-	    bool color,
-	    const char *dumpfile = NULL,
-	    int trim3 = 0,
-	    int trim5 = 0);
+		bool color,
+		int trim3 = 0,
+		int trim5 = 0);
 	
 	virtual ~VectorPatternSource() { }
 	
@@ -536,6 +510,10 @@ public:
 
 private:
 
+	pair<bool, int> nextBatchImpl(
+		PerThreadReadBuf& pt,
+		bool batch_a);
+
 	bool color_;                      // colorspace?
 	size_t cur_;                      // index for first read of next batch
 	bool paired_;                     // whether reads are paired
@@ -553,12 +531,11 @@ private:
 class CFilePatternSource : public TrimmingPatternSource {
 public:
 	CFilePatternSource(
-	    const vector<string>& infiles,
-	    const vector<string>* qinfiles,
-	    const char *dumpfile = NULL,
-	    int trim3 = 0,
+		const vector<string>& infiles,
+		const vector<string>* qinfiles,
+		int trim3 = 0,
 	    int trim5 = 0) :
-		TrimmingPatternSource(dumpfile, trim3, trim5),
+		TrimmingPatternSource( trim3, trim5),
 		infiles_(infiles),
 		filecur_(0),
 		fp_(NULL),
@@ -583,13 +560,21 @@ public:
 
 	virtual ~CFilePatternSource() {
 		if(is_open_) {
-			assert(fp_ != NULL);
-			fclose(fp_);
-			fp_ = NULL;
-			if(qfp_ != NULL) {
+			if (compressed_) {
+				gzclose(zfp_);
+				zfp_ = NULL;
+			}
+			else if (fp_ != stdin) {
+				fclose(fp_);
+				fp_ = NULL;
+			}
+			if(qfp_ != NULL && qfp_ != stdin) {
 				fclose(qfp_);
 				qfp_ = NULL;
 			}
+			assert(zfp_ == NULL);
+			assert(fp_ == NULL || fp_ == stdin);
+			assert(qfp_ == NULL || qfp_ == stdin);
 		}
 	}
 
@@ -625,7 +610,8 @@ protected:
 	 */
 	virtual std::pair<bool, int> nextBatchFromFile(
 		PerThreadReadBuf& pt,
-		bool batch_a) = 0;
+		bool batch_a,
+		size_t read_idx) = 0;
 
 	/**
 	 * Reset state to handle a fresh file
@@ -636,6 +622,31 @@ protected:
 	 * Open the next file in the list of input files.
 	 */
 	void open();
+
+	int getc_wrapper() {
+		return compressed_ ? gzgetc(zfp_) : getc_unlocked(fp_);
+	}
+
+	int ungetc_wrapper(int c) {
+		return compressed_ ? gzungetc(c, zfp_) : ungetc(c, fp_);
+	}
+
+	bool is_gzipped_file(const std::string& filename) {
+		struct stat s;
+		if (stat(filename.c_str(), &s) != 0) {
+			perror("stat");
+		}
+		else {
+			if (S_ISFIFO(s.st_mode))
+				return true;
+		}
+		size_t pos = filename.find_last_of(".");
+		std::string ext = (pos == std::string::npos) ? "" : filename.substr(pos + 1);
+		if (ext == "" || ext == "gz" || ext == "Z") {
+			return true;
+		}
+		return false;
+	}
 	
 	vector<string> infiles_; /// filenames for read files
 	vector<string> qinfiles_; /// filenames for quality files
@@ -643,10 +654,19 @@ protected:
 	size_t filecur_;   /// index into infiles_ of next file to read
 	FILE *fp_; /// read file currently being read from
 	FILE *qfp_; /// quality file currently being read from
+    gzFile zfp_;
 	bool is_open_; /// whether fp_ is currently open
 	bool first_;
 	char buf_[64*1024]; /// file buffer for sequences
 	char qbuf_[64*1024]; /// file buffer for qualities
+    bool compressed_;
+
+private:
+
+	pair<bool, int> nextBatchImpl(
+		PerThreadReadBuf& pt,
+		bool batch_a);
+
 };
 
 /**
@@ -659,20 +679,18 @@ public:
 
 	FastaPatternSource(
 		const vector<string>& infiles,
-	    const vector<string>* qinfiles,
-	    bool color,
-	    const char *dumpfile = NULL,
-	    int trim3 = 0,
-	    int trim5 = 0,
-	    bool solexa64 = false,
-	    bool phred64 = false,
-	    bool intQuals = false) :
+		const vector<string>* qinfiles,
+		bool color,
+		int trim3 = 0,
+		int trim5 = 0,
+		bool solexa64 = false,
+		bool phred64 = false,
+		bool intQuals = false) :
 		CFilePatternSource(
 			infiles,
 			qinfiles,
-		    dumpfile,
 			trim3,
-		    trim5),
+			trim5),
 		first_(true),
 		color_(color),
 		solexa64_(solexa64),
@@ -700,7 +718,8 @@ protected:
 	 */
 	virtual std::pair<bool, int> nextBatchFromFile(
 		PerThreadReadBuf& pt,
-		bool batch_a);
+		bool batch_a,
+		size_t read_idx);
 
 	/**
 	 * Reset state to handle a fresh file
@@ -748,18 +767,16 @@ public:
 	TabbedPatternSource(
 		const vector<string>& infiles,
 		bool secondName,  // whether it's --12/--tab5 or --tab6
-	    bool color,
-	    const char *dumpfile = NULL,
-	    int trim3 = 0,
-	    int trim5 = 0,
-	    bool solQuals = false,
-	    bool phred64Quals = false,
-	    bool intQuals = false) :
+		bool color,
+		int trim3 = 0,
+		int trim5 = 0,
+		bool solQuals = false,
+		bool phred64Quals = false,
+		bool intQuals = false) :
 		CFilePatternSource(
 			infiles,
 			NULL,
-		    dumpfile,
-		    trim3,
+			trim3,
 			trim5),
 		color_(color),
 		solQuals_(solQuals),
@@ -778,7 +795,8 @@ protected:
 	 */
 	virtual std::pair<bool, int> nextBatchFromFile(
 		PerThreadReadBuf& pt,
-		bool batch_a);
+		bool batch_a,
+		size_t read_idx);
 	
 	/**
 	 * Dump a FASTQ-style record for the read.
@@ -810,12 +828,10 @@ public:
 	FastaContinuousPatternSource(
 			const vector<string>& infiles,
 			size_t length,
-			size_t freq,
-			const char *dumpfile = NULL) :
+			size_t freq) :
 		CFilePatternSource(
 			infiles,
 			NULL,
-		    dumpfile,
 			0,
 			0),
 		length_(length),
@@ -847,7 +863,8 @@ protected:
 	 */
 	virtual std::pair<bool, int> nextBatchFromFile(
 		PerThreadReadBuf& pt,
-		bool batch_a);
+		bool batch_a,
+		size_t read_idx);
 
 	/**
 	 * Reset state to be read for the next file.
@@ -886,24 +903,24 @@ class FastqPatternSource : public CFilePatternSource {
 public:
 	FastqPatternSource(
 		const vector<string>& infiles,
-	    bool color,
-	    const char *dumpfile = NULL,
-	    int trim3 = 0,
-	    int trim5 = 0,
-	    bool solexa_quals = false,
-	    bool phred64Quals = false,
-	    bool integer_quals = false,
-	    uint32_t skip = 0) :
+		bool color,
+		int trim3 = 0,
+		int trim5 = 0,
+		bool solexa_quals = false,
+		bool phred64Quals = false,
+		bool integer_quals = false,
+		bool interleaved = false,
+		uint32_t skip = 0) :
 		CFilePatternSource(
 			infiles,
 			NULL,
-		    dumpfile,
-		    trim3,
+			trim3,
 			trim5),
 		first_(true),
 		solQuals_(solexa_quals),
 		phred64Quals_(phred64Quals),
 		intQuals_(integer_quals),
+		interleaved_(interleaved),
 		color_(color) { }
 	
 	virtual void reset() {
@@ -927,7 +944,8 @@ protected:
 	 */
 	virtual pair<bool, int> nextBatchFromFile(
 		PerThreadReadBuf& pt,
-		bool batch_a);
+		bool batch_a,
+		size_t read_idx);
 
 	virtual void resetForNextFile() {
 		first_ = true;
@@ -947,6 +965,7 @@ private:
 	bool solQuals_;
 	bool phred64Quals_;
 	bool intQuals_;
+	bool interleaved_;
 	bool color_;
 };
 
@@ -961,14 +980,12 @@ public:
 
 	RawPatternSource(
 		const vector<string>& infiles,
-	    bool color,
-	    const char *dumpfile = NULL,
+		bool color,
 		int trim3 = 0,
-	    int trim5 = 0) :
+		int trim5 = 0) :
 		CFilePatternSource(
 			infiles,
 			NULL,
-		    dumpfile,
 			trim3,
 			trim5),
 		first_(true),
@@ -991,7 +1008,8 @@ protected:
 	 */
 	virtual std::pair<bool, int> nextBatchFromFile(
 		PerThreadReadBuf& pt,
-		bool batch_a);
+		bool batch_a,
+		size_t read_idx);
 
 	virtual void resetForNextFile() {
 		first_ = true;
@@ -1308,6 +1326,8 @@ public:
 		// Free the vector
 		delete composers;
 	}
+
+	virtual ~PatternSourcePerThreadFactory() {}
 	
 private:
 	/// Container for obtaining paired reads from PatternSources
